// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancyexception.h 87 2007-12-18 17:11:32Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Global exception class stuff.
** 
*****************************************************************************/

#ifndef FANCY_EXCEPTION_H
#define FANCY_EXCEPTION_H

#include <fancybaselog.h>
#include <exception>

namespace Fancy
{
  class FANCYBASE_API Exception : public std::exception
    { 
    public:
      Exception (int facility,
                 int priority,
                 const char *fmt, ...) throw()
#ifdef __GNUG__
        __attribute__((format(printf,4,5)))
#endif
        ;

      Exception (const Exception &) throw();
      Exception (const std::exception &) throw();
      Exception &operator= (const Exception &) throw();

      ~Exception() throw();

      static void log(const std::exception &e) throw();

      virtual void log() const throw();
      const char *what() const throw();
      fancylog_service *getService() const throw();
      int getFacility() const throw();
      int getPriority() const throw();

    protected:
      Exception (fancylog_service *service,
                 int facility,
                 int priority ) throw();

      void appendMessageV(const char *fmt, va_list ap) throw();
#ifdef __GNUG__
      __attribute__((format(printf,2,3)))
#endif
      void appendMessage(const char *fmt, ...) throw();

    private:
      fancylog_service *svc;
      int fac;
      int pri;
      char *msg;
    };

  class FANCYBASE_API CException : public Exception
    { 
    public:
      CException (int facility,
                  int priority,
                  const char *fmt, ...) throw()
#ifdef __GNUG__
      __attribute__((format(printf,4,5)))
#endif
        ;

      CException (int facility,
                  int priority,
                  int _err,
                  const char *fmt, ...) throw()
#ifdef __GNUG__
      __attribute__((format(printf,5,6)))
#endif
        ;

      ~CException() throw();

      int getErrno() const throw();
      const char *getStrError() const throw();

      virtual bool isInterrupt() const throw();

    protected:
      CException (fancylog_service *service,
                  int facility,
                  int priority) throw();

      CException (fancylog_service *service,
                  int facility,
                  int priority,
                  int _err) throw();

      void appendCMessageV(const char *msg, va_list ap) throw ();
#ifdef __GNUG__
      __attribute__((format(printf,2,3)))
#endif
      void appendCMessage(const char *msg, ...) throw ();

    private:
      int err;
#ifdef WIN32
      char *strerr;
#endif
    };

}; // namespace Fancy

#endif // ! FANCY_EXCEPTION_H
