// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancyfilehandle.h 93 2008-01-23 10:36:56Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Wrapper class for a file handle.
** 
*****************************************************************************/

#ifndef FANCYBASE_FILE_HANDLE_H
#define FANCYBASE_FILE_HANDLE_H

#ifdef WIN32
#define POLLIN      0x0001
#define POLLPRI     0x0002
#define POLLOUT     0x0004
#define POLLERR     0x0008
#define POLLHUP     0x0010
#define POLLNVAL    0x0020
#else
#include <sys/poll.h>
#endif
#include <fancybase.h>

namespace Fancy {
#ifdef WIN32
      typedef void *file_handle_t;
#else
      typedef int file_handle_t;
#endif

  class FANCYBASE_API FileHandle
  {
    file_handle_t fd;

  public:
    enum Mode { ReadOnly, WriteOnly, ReadWrite };
    enum InheritFlag { InheritNone, InheritRead, InheritWrite, InheritBoth };
    enum StdHandle { StdIn, StdOut, StdErr };
    
#ifdef WIN32
    FileHandle(unsigned buf_size);
#endif

    FileHandle();
   
    FileHandle(StdHandle stdHandle);
   
    ~FileHandle();

    bool isOpen() const;
    file_handle_t getHandle() const;

    void close();

    void open(file_handle_t fd, Mode mode);

    void openFile(const char *path, Mode mode);

    int poll (short *events, int timeout);

    int read (void *buf, int count);
    int write (const void *buf, int count);

    static int poll (FileHandle *handles, short *events, unsigned n_handles, int timeout);

    static void openPipe(FileHandle *readHandle, FileHandle *writeHandle, InheritFlag flag);
  };

}

#endif

