/****************************************************************************
** $Id: fancytimeval.h 87 2007-12-18 17:11:32Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas/EV-i
**
** Convenience C++ operators on struct timeval from sys/time.h.
**
*****************************************************************************/

#ifndef FANCY_TIMEVAL_H
#define FANCY_TIMEVAL_H 1

#include <sys/time.h>

// convenience operators on struct timeval, mainly a cut'n'paste
// from the C-macros in sys/time.h Since these macros are not always
// included, we make C++-inlines from them and put it here.
// (Under glibc-2.2, you have to #define _BSD_SOURCE cf. to gettimeofday(2)
//  and under other UNICES, I suspect them to be completely missing.)

inline bool operator < (const struct timeval &a, const struct timeval &b)
{
  return 
    (a.tv_sec == b.tv_sec)  ? 
    (a.tv_usec < b.tv_usec) :
    (a.tv_sec < b.tv_sec)   ;
}

inline bool operator > (const struct timeval &a, const struct timeval &b)
{
  return 
    (a.tv_sec == b.tv_sec)  ? 
    (a.tv_usec > b.tv_usec) :
    (a.tv_sec > b.tv_sec)   ;
}

inline bool operator >= (const struct timeval &a, const struct timeval &b)
{
  return 
    (a.tv_sec == b.tv_sec)  ? 
    (a.tv_usec >= b.tv_usec) :
    (a.tv_sec > b.tv_sec)   ;
}

inline bool operator <= (const struct timeval &a, const struct timeval &b)
{
  return 
    (a.tv_sec == b.tv_sec)  ? 
    (a.tv_usec <= b.tv_usec) :
    (a.tv_sec < b.tv_sec)   ;
}

inline struct timeval &operator+=(struct timeval &a, const struct timeval &b)
{
  a.tv_sec  += b.tv_sec;
  a.tv_usec += b.tv_usec;

  if (a.tv_usec >= 1000000) {
    ++a.tv_sec;
    a.tv_usec -= 1000000;
  }
  return a;
}

inline struct timeval &operator -= (struct timeval &a, const struct timeval &b)
{
  a.tv_sec -= b.tv_sec;
  a.tv_usec -= b.tv_usec;
  if (a.tv_usec < 0) {
    --a.tv_sec;
    a.tv_usec += 1000000;
  }
  return a;
}

inline int tv_to_msec (const struct timeval &a)
{
  return a.tv_sec*1000 + (a.tv_usec+500) / 1000;
}

inline void tv_set_msec (struct timeval &a, unsigned m)
{
  a.tv_sec=m/1000;
  a.tv_usec=(m%1000)*1000;
}

#endif // ! FANCYSOCK_TIMEVAL_H
